@@ -84,7 +84,7 @@
 /* Handle module flags. */
 
 #define MOD_HAS_DELAYED_DTORS   0x0001  /* It has a class with a delayed dtor. */
-#define MOD_IS_CONSOLIDATED     0x0002  /* It is a consolidated module. */
+#define MOD_IS_UNUSED           0x0002  /* This flag is unused. */
 #define MOD_IS_COMPOSITE        0x0004  /* It is a composite module. */
 #define MOD_IS_TRANSFORMED      0x0008  /* It's types have been transformed. */
 #define MOD_USE_ARG_NAMES       0x0010  /* Use real argument names. */
@@ -95,14 +95,12 @@
 #define MOD_SUPER_INIT_UNDEF    0x0000  /* Calling super().__init__() is undefined. */
 #define MOD_SUPER_INIT_MASK     0x0180  /* The mask for the above flags. */
 #define MOD_SETTING_IMPORTS     0x0200  /* Imports are being set. */
+#define MOD_PY_SSIZE_T_CLEAN    0x0400  /* #define PY_SSIZE_T_CLEAN. */
 
 #define hasDelayedDtors(m)  ((m)->modflags & MOD_HAS_DELAYED_DTORS)
 #define setHasDelayedDtors(m)   ((m)->modflags |= MOD_HAS_DELAYED_DTORS)
-#define isConsolidated(m)   ((m)->modflags & MOD_IS_CONSOLIDATED)
-#define setIsConsolidated(m)    ((m)->modflags |= MOD_IS_CONSOLIDATED)
 #define isComposite(m)      ((m)->modflags & MOD_IS_COMPOSITE)
 #define setIsComposite(m)   ((m)->modflags |= MOD_IS_COMPOSITE)
-#define isContainer(m)      ((m)->modflags & (MOD_IS_CONSOLIDATED | MOD_IS_COMPOSITE))
 #define setIsTransformed(m) ((m)->modflags |= MOD_IS_TRANSFORMED)
 #define isTransformed(m)    ((m)->modflags & MOD_IS_TRANSFORMED)
 #define setUseArgNames(m)   ((m)->modflags |= MOD_USE_ARG_NAMES)
@@ -118,6 +116,8 @@
 #define settingImports(m)   ((m)->modflags & MOD_SETTING_IMPORTS)
 #define setSettingImports(m)    ((m)->modflags |= MOD_SETTING_IMPORTS)
 #define resetSettingImports(m)  ((m)->modflags &= ~MOD_SETTING_IMPORTS)
+#define setPY_SSIZE_T_CLEAN(m)  ((m)->modflags |= MOD_PY_SSIZE_T_CLEAN)
+#define isPY_SSIZE_T_CLEAN(m)   ((m)->modflags & MOD_PY_SSIZE_T_CLEAN)
 
 
 /* Handle section flags. */
@@ -304,11 +304,12 @@
 #define ENUM_NO_SCOPE       0x00000200  /* Omit the member scopes. */
 #define ENUM_NEEDS_ENUM     0x00000400  /* The module needs it. */
 #define ENUM_SCOPED         0x00000800  /* A C++0x11 scoped enum. */
-#define ENUM_MASK           0x00003000  /* The enum base type mask. */
+#define ENUM_MASK           0x00007000  /* The enum base type mask. */
 #define ENUM_ENUM           0x00000000  /* The base type is Enum. */
 #define ENUM_FLAG           0x00001000  /* The base type is Flag. */
 #define ENUM_INT_ENUM       0x00002000  /* The base type is IntEnum. */
 #define ENUM_INT_FLAG       0x00003000  /* The base type is IntFlag. */
+#define ENUM_UINT_ENUM      0x00004000  /* The base type is IntEnum with unsigned members. */
 
 #define isProtectedEnum(e)  ((e)->enumflags & SECT_IS_PROT)
 #define setIsProtectedEnum(e)   ((e)->enumflags |= SECT_IS_PROT)
@@ -332,18 +333,17 @@
 #define setIsEnumIntEnum(e) ((e)->enumflags |= ENUM_INT_ENUM)
 #define isEnumIntFlag(e)    (((e)->enumflags & ENUM_MASK) == ENUM_INT_FLAG)
 #define setIsEnumIntFlag(e) ((e)->enumflags |= ENUM_INT_FLAG)
+#define isEnumUIntEnum(e)   (((e)->enumflags & ENUM_MASK) == ENUM_UINT_ENUM)
+#define setIsEnumUIntEnum(e)    ((e)->enumflags |= ENUM_UINT_ENUM)
 
 
 /* Handle hierarchy flags. */
 
 #define HIER_BEING_SET      0x0001      /* The MRO is being set. */
-#define IN_A_DIAMOND        0x0002      /* The class is in a diamond. */
 
 #define hierBeingSet(m)     ((m)->mroflags & HIER_BEING_SET)
 #define setHierBeingSet(m)  ((m)->mroflags |= HIER_BEING_SET)
 #define resetHierBeingSet(m)    ((m)->mroflags &= ~HIER_BEING_SET)
-#define inADiamond(m)       ((m)->mroflags & IN_A_DIAMOND)
-#define setInADiamond(m)    ((m)->mroflags |= IN_A_DIAMOND)
 
 
 /* Handle overload flags.  These are combined with the section flags. */
@@ -793,13 +793,6 @@ typedef struct _qualDef {
 } qualDef;
 
 
-/* A platform. */
-typedef struct _platformDef {
-    struct _qualDef *qualifier;         /* The platform qualifier. */
-    struct _platformDef *next;          /* Next in the list. */
-} platformDef;
-
-
 /* A scoped name. */
 typedef struct _scopedNameDef {
     char *name;                         /* The name. */
@@ -1342,7 +1335,6 @@ typedef struct {
     virtErrorHandler *errorhandlers;    /* The list of virtual error handlers. */
     codeBlockList *exphdrcode;          /* Exported header code. */
     codeBlockList *exptypehintcode;     /* Exported type hint code. */
-    codeBlockList *docs;                /* Documentation. */
     classDef *qobject_cd;               /* QObject class, NULL if none. */
     int genc;                           /* Set if we are generating C code. */
     struct _stringList *plugins;        /* The list of plugins. */
@@ -1413,7 +1405,9 @@ scopedNameDef *copyScopedName(scopedNameDef *);
 void appendScopedName(scopedNameDef **,scopedNameDef *);
 scopedNameDef *text2scopePart(char *text);
 void freeScopedName(scopedNameDef *);
+char *scopedNameToString(scopedNameDef *name);
 void appendToClassList(classList **,classDef *);
+void appendCodeBlock(codeBlockList **headp, codeBlock *cb);
 void appendCodeBlockList(codeBlockList **headp, codeBlockList *cbl);
 void prcode(FILE *fp, const char *fmt, ...);
 void prCopying(FILE *fp, moduleDef *mod, const char *comment);
@@ -1429,9 +1423,11 @@ int isVoidReturnSlot(memberDef *md);
 int isNumberSlot(memberDef *md);
 int isInplaceNumberSlot(memberDef *md);
 int isRichCompareSlot(memberDef *md);
-mappedTypeDef *allocMappedType(sipSpec *pt, argDef *type);
+mappedTypeDef *allocMappedType(sipSpec *pt, argDef *type, int use_name);
 void appendString(stringList **headp, const char *s);
-void appendTypeStrings(scopedNameDef *ename, signatureDef *patt, signatureDef *src, signatureDef *known, scopedNameDef **names, scopedNameDef **values);
+void templateExpansions(signatureDef *patt, signatureDef *src,
+        signatureDef *declared_names, scopedNameDef **names,
+        scopedNameDef **values);
 codeBlockList *templateCode(sipSpec *pt, ifaceFileList **used,
         codeBlockList *ocbl, scopedNameDef *names, scopedNameDef *values);
 ifaceFileDef *findIfaceFile(sipSpec *pt, moduleDef *mod,
@@ -1445,7 +1441,6 @@ nameDef *cacheName(sipSpec *pt, const char *name);
 scopedNameDef *encodedTemplateName(templateDef *td);
 memberDef *findMethod(classDef *cd, const char *name);
 typeHintDef *newTypeHint(char *raw_hint);
-int isPyKeyword(const char *word);
 char *templateString(const char *src, scopedNameDef *names,
         scopedNameDef *values);
 void dsCtor(sipSpec *pt, classDef *cd, ctorDef *ct, FILE *fp);
@@ -1462,6 +1457,7 @@ void normaliseArgs(signatureDef *sd);
 void restoreArgs(signatureDef *sd);
 void initialiseLexer(void);
 int usedInCode(codeBlockList *cbl, const char *str);
+int selectedQualifier(stringList *needed_qualifiers, qualDef *qd);
 
 
 /* These are only here because bison publically references them. */
@@ -1604,6 +1600,7 @@ typedef struct _moduleCfg {
     KwArgs kwargs;
     const char *name;
     int use_arg_names;
+    int py_ssize_t_clean;
     int use_limited_api;
     int all_raise_py_exc;
     int call_super_init;
