@@ -1,7 +1,7 @@
 /*
  * The PEP 484 type hints generator for SIP.
  *
- * Copyright (c) 2018 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2022 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -79,15 +79,13 @@ static enumDef *lookupEnum(sipSpec *pt, const char *name, classDef *scope_cd,
 static mappedTypeDef *lookupMappedType(sipSpec *pt, const char *name);
 static classDef *lookupClass(sipSpec *pt, const char *name,
         classDef *scope_cd);
-static classDef *getClassImplementation(sipSpec *pt, classDef *cd);
-static mappedTypeDef *getMappedTypeImplementation(sipSpec *pt,
-        mappedTypeDef *mtd);
 static void maybeAnyObject(const char *hint, int pep484, FILE *fp);
 static void strip_leading(char **startp, char *end);
 static void strip_trailing(char *start, char **endp);
 static typeHintNodeDef *flatten_unions(typeHintNodeDef *nodes);
 static typeHintNodeDef *copyTypeHintNode(sipSpec *pt, typeHintDef *thd,
         int out);
+static int isPyKeyword(const char *word);
 
 
 /*
@@ -104,10 +102,13 @@ void generateTypeHints(sipSpec *pt, moduleDef *mod, const char *pyiFile)
     /* Write the header. */
     fprintf(fp,
 "# The PEP 484 type hints stub file for the %s module.\n"
+        , mod->name);
+
+    if (sipVersionStr != NULL)
+        fprintf(fp,
 "#\n"
 "# Generated by SIP %s\n"
-        , mod->name
-        , sipVersionStr);
+            , sipVersionStr);
 
     prCopying(fp, mod, "#");
 
@@ -155,10 +156,16 @@ static void pyiModule(sipSpec *pt, moduleDef *mod, FILE *fp)
      * Generate the imports. Note that we assume the super-types are the
      * standard SIP ones.
      */
+    if (abiVersion >= ABI_13_0)
+        fprintf(fp,
+"import enum\n"
+            );
+
     fprintf(fp,
 "import typing\n"
-"import sip\n"
-        );
+"\n"
+"import %s\n"
+        , (sipName != NULL) ? sipName : "sip");
 
     first = TRUE;
 
@@ -194,40 +201,33 @@ static void pyiModule(sipSpec *pt, moduleDef *mod, FILE *fp)
 
     for (cd = pt->classes; cd != NULL; cd = cd->next)
     {
-        classDef *impl;
-
         if (cd->iff->module != mod)
             continue;
 
         if (isExternal(cd))
             continue;
 
-        impl = getClassImplementation(pt, cd);
+        if (cd->no_typehint)
+            continue;
 
-        if (impl != NULL)
-        {
-            if (impl->no_typehint)
-                continue;
+        /* Only handle non-nested classes here. */
+        if (cd->ecd != NULL)
+            continue;
 
-            /* Only handle non-nested classes here. */
-            if (impl->ecd != NULL)
-                continue;
+        /* We can't handle extenders. */
+        if (cd->real != NULL)
+            continue;
 
-            pyiClass(pt, mod, impl, &defined, 0, fp);
-        }
+        pyiClass(pt, mod, cd, &defined, 0, fp);
     }
 
     for (mtd = pt->mappedtypes; mtd != NULL; mtd = mtd->next)
     {
-        mappedTypeDef *impl;
-
         if (mtd->iff->module != mod)
             continue;
 
-        impl = getMappedTypeImplementation(pt, mtd);
-
-        if (impl != NULL && impl->pyname != NULL)
-            pyiMappedType(pt, mod, impl, &defined, 0, fp);
+        if (mtd->pyname != NULL)
+            pyiMappedType(pt, mod, mtd, &defined, 0, fp);
     }
 
     pyiVars(pt, mod, NULL, defined, 0, fp);
@@ -287,139 +287,129 @@ static void pyiClass(sipSpec *pt, moduleDef *mod, classDef *cd,
     memberDef *md;
     propertyDef *pd;
 
-    separate(TRUE, indent, fp);
-    prIndent(indent, fp);
-    fprintf(fp, "class %s(", cd->pyname->text);
-
-    if (cd->supers != NULL)
-    {
-        classList *cl;
-
-        for (cl = cd->supers; cl != NULL; cl = cl->next)
-        {
-            if (cl != cd->supers)
-                fprintf(fp, ", ");
-
-            prClassRef(cl->cd, mod, *defined, TRUE, fp);
-        }
-    }
-    else if (cd->supertype != NULL)
-    {
-        fprintf(fp, "%s", cd->supertype->text);
-    }
-    else if (cd->iff->type == namespace_iface)
-    {
-        fprintf(fp, "sip.simplewrapper");
-    }
-    else
-    {
-        fprintf(fp, "sip.wrapper");
-    }
-
-    /* See if there is anything in the class body. */
     nr_overloads = 0;
 
-    for (ct = cd->ctors; ct != NULL; ct = ct->next)
+    if (!isHiddenNamespace(cd))
     {
-        if (isPrivateCtor(ct))
-            continue;
+        separate(TRUE, indent, fp);
+        prIndent(indent, fp);
+        fprintf(fp, "class %s(", cd->pyname->text);
 
-        if (ct->no_typehint)
-            continue;
-
-        if (!inDefaultAPI(pt, ct->api_range))
-            continue;
-
-        ++nr_overloads;
-    }
-
-    no_body = (cd->typehintcode == NULL && nr_overloads == 0);
-
-    if (no_body)
-    {
-        overDef *od;
-
-        for (od = cd->overs; od != NULL; od = od->next)
+        if (cd->supers != NULL)
         {
-            if (isPrivate(od))
-                continue;
+            classList *cl;
 
-            if (od->no_typehint)
-                continue;
-
-            if (inDefaultAPI(pt, od->api_range))
+            for (cl = cd->supers; cl != NULL; cl = cl->next)
             {
+                if (cl != cd->supers)
+                    fprintf(fp, ", ");
+
+                prClassRef(cl->cd, mod, *defined, TRUE, fp);
+            }
+        }
+        else if (cd->supertype != NULL)
+        {
+            fprintf(fp, "%s", cd->supertype->text);
+        }
+        else
+        {
+            fprintf(fp, "%s.%swrapper", (sipName != NULL ? sipName : "sip"),
+                    (cd->iff->type == namespace_iface ? "simple" : ""));
+        }
+
+        /* See if there is anything in the class body. */
+        for (ct = cd->ctors; ct != NULL; ct = ct->next)
+        {
+            if (isPrivateCtor(ct))
+                continue;
+
+            if (ct->no_typehint)
+                continue;
+
+            ++nr_overloads;
+        }
+
+        no_body = (cd->typehintcode == NULL && nr_overloads == 0);
+
+        if (no_body)
+        {
+            overDef *od;
+
+            for (od = cd->overs; od != NULL; od = od->next)
+            {
+                if (isPrivate(od))
+                    continue;
+
+                if (od->no_typehint)
+                    continue;
+
                 no_body = FALSE;
                 break;
             }
         }
-    }
 
-    if (no_body)
-    {
-        enumDef *ed;
-
-        for (ed = pt->enums; ed != NULL; ed = ed->next)
+        if (no_body)
         {
-            if (ed->no_typehint)
-                continue;
+            enumDef *ed;
 
-            if (ed->ecd == cd)
+            for (ed = pt->enums; ed != NULL; ed = ed->next)
             {
-                no_body = FALSE;
-                break;
+                if (ed->no_typehint)
+                    continue;
+
+                if (ed->ecd == cd)
+                {
+                    no_body = FALSE;
+                    break;
+                }
             }
         }
-    }
 
-    if (no_body)
-    {
-        for (nested = pt->classes; nested != NULL; nested = nested->next)
+        if (no_body)
         {
-            if (nested->no_typehint)
-                continue;
-
-            if (nested->ecd == cd)
+            for (nested = pt->classes; nested != NULL; nested = nested->next)
             {
-                no_body = FALSE;
-                break;
+                if (nested->no_typehint)
+                    continue;
+
+                if (nested->ecd == cd)
+                {
+                    no_body = FALSE;
+                    break;
+                }
             }
         }
-    }
 
-    if (no_body)
-    {
-        varDef *vd;
-
-        for (vd = pt->vars; vd != NULL; vd = vd->next)
+        if (no_body)
         {
-            if (vd->no_typehint)
-                continue;
+            varDef *vd;
 
-            if (vd->ecd == cd)
+            for (vd = pt->vars; vd != NULL; vd = vd->next)
             {
-                no_body = FALSE;
-                break;
+                if (vd->no_typehint)
+                    continue;
+
+                if (vd->ecd == cd)
+                {
+                    no_body = FALSE;
+                    break;
+                }
             }
         }
+
+        fprintf(fp, "):%s\n", (no_body ? " ..." : ""));
+
+        ++indent;
+
+        pyiTypeHintCode(cd->typehintcode, indent, fp);
     }
 
-    fprintf(fp, "):%s\n", (no_body ? " ..." : ""));
-
-    ++indent;
-
-    pyiTypeHintCode(cd->typehintcode, indent, fp);
-
     pyiEnums(pt, mod, cd->iff, *defined, indent, fp);
 
     /* Handle any nested classes. */
     for (nested = pt->classes; nested != NULL; nested = nested->next)
-    {
-        classDef *impl = getClassImplementation(pt, nested);
-
-        if (impl != NULL && impl->ecd == cd && !impl->no_typehint)
-            pyiClass(pt, mod, impl, defined, indent, fp);
-    }
+        if (nested->ecd == cd && !nested->no_typehint)
+            pyiClass(pt, mod, nested, defined, indent, fp);
 
     pyiVars(pt, mod, cd, *defined, indent, fp);
 
@@ -435,9 +425,6 @@ static void pyiClass(sipSpec *pt, moduleDef *mod, classDef *cd,
         if (ct->no_typehint)
             continue;
 
-        if (!inDefaultAPI(pt, ct->api_range))
-            continue;
-
         overloaded = (nr_overloads > 1);
 
         first = separate(first, indent, fp);
@@ -460,7 +447,8 @@ static void pyiClass(sipSpec *pt, moduleDef *mod, classDef *cd,
 
         first = separate(first, indent, fp);
 
-        pyiCallable(pt, mod, md, cd->overs, TRUE, *defined, indent, fp);
+        pyiCallable(pt, mod, md, cd->overs, !isHiddenNamespace(cd), *defined,
+                indent, fp);
     }
 
     for (pd = cd->properties; pd != NULL; pd = pd->next)
@@ -484,11 +472,14 @@ static void pyiClass(sipSpec *pt, moduleDef *mod, classDef *cd,
         }
     }
 
-    /*
-     * Keep track of what has been defined so that forward references are no
-     * longer required.
-     */
-    appendToIfaceFileList(defined, cd->iff);
+    if (!isHiddenNamespace(cd))
+    {
+        /*
+         * Keep track of what has been defined so that forward references are
+         * no longer required.
+         */
+        appendToIfaceFileList(defined, cd->iff);
+    }
 }
 
 
@@ -627,8 +618,25 @@ static void pyiEnums(sipSpec *pt, moduleDef *mod, ifaceFileDef *scope,
 
         if (ed->pyname != NULL)
         {
+            const char *super = "int";
+
             prIndent(indent, fp);
-            fprintf(fp, "class %s(int): ...\n", ed->pyname->text);
+
+            if (abiVersion >= ABI_13_0)
+            {
+                if (isEnumEnum(ed))
+                    super = "enum.Enum";
+                else if (isEnumFlag(ed))
+                    super = "enum.Flag";
+                else if (isEnumIntEnum(ed) || isEnumUIntEnum(ed))
+                    super = "enum.IntEnum";
+                else if (isEnumIntFlag(ed))
+                    super = "enum.IntFlag";
+            }
+
+            fprintf(fp, "class %s(%s):\n", ed->pyname->text, super);
+
+            ++indent;
         }
 
         for (emd = ed->members; emd != NULL; emd = emd->next)
@@ -640,12 +648,15 @@ static void pyiEnums(sipSpec *pt, moduleDef *mod, ifaceFileDef *scope,
             fprintf(fp, "%s = ... # type: ", emd->pyname->text);
 
             if (ed->pyname != NULL)
-                prEnumRef(ed, mod, defined, TRUE, fp);
+                prScopedPythonName(fp, ed->ecd, ed->pyname->text);
             else
                 fprintf(fp, "int");
 
             fprintf(fp, "\n");
         }
+
+        if (ed->pyname != NULL)
+            --indent;
     }
 }
 
@@ -704,9 +715,6 @@ static void pyiCallable(sipSpec *pt, moduleDef *mod, memberDef *md,
         if (od->no_typehint)
             continue;
 
-        if (!inDefaultAPI(pt, od->api_range))
-            continue;
-
         ++nr_overloads;
     }
 
@@ -724,9 +732,6 @@ static void pyiCallable(sipSpec *pt, moduleDef *mod, memberDef *md,
         if (od->no_typehint)
             continue;
 
-        if (!inDefaultAPI(pt, od->api_range))
-            continue;
-
         overloaded = (nr_overloads > 1);
 
         pyiOverload(pt, mod, od, overloaded, is_method, defined, indent, TRUE,
@@ -934,7 +939,7 @@ void prDefaultValue(argDef *ad, int in_str, FILE *fp)
 static void pyiType(sipSpec *pt, moduleDef *mod, argDef *ad, int out,
         ifaceFileList *defined, int pep484, FILE *fp)
 {
-    const char *type_name;
+    const char *type_name, *sip_name;
     typeHintDef *thd;
 
     /* Use any explicit type hint unless the argument is constrained. */
@@ -946,35 +951,24 @@ static void pyiType(sipSpec *pt, moduleDef *mod, argDef *ad, int out,
         return;
     }
 
-    /* For classes and mapped types we need the default implementation. */
-    if (ad->atype == class_type || ad->atype == mapped_type)
-    {
-        classDef *cd = ad->u.cd;
-        mappedTypeDef *mtd = ad->u.mtd;
-
-        getDefaultImplementation(pt, ad->atype, &cd, &mtd);
-
-        if (cd != NULL)
-        {
-            prClassRef(cd, mod, defined, pep484, fp);
-        }
-        else
-        {
-            /*
-             * This should never happen as it should have been picked up when
-             * generating code - but maybe we haven't been asked to generate
-             * code.
-             */
-            fprintf(fp, anyObject(pep484));
-        }
-
-        return;
-    }
-
     type_name = NULL;
 
+    sip_name = (sipName != NULL ? sipName : "sip");
+
     switch (ad->atype)
     {
+    case class_type:
+        prClassRef(ad->u.cd, mod, defined, pep484, fp);
+        break;
+
+    case mapped_type:
+        /*
+         * This should never happen as it should have been picked up when
+         * generating code - but maybe we haven't been asked to generate code.
+         */
+        fprintf(fp, anyObject(pep484));
+        break;
+
     case enum_type:
         if (ad->u.ed->pyname != NULL)
             prEnumRef(ad->u.ed, mod, defined, pep484, fp);
@@ -989,16 +983,15 @@ static void pyiType(sipSpec *pt, moduleDef *mod, argDef *ad, int out,
 
     case struct_type:
     case void_type:
-        type_name = "sip.voidptr";
-        break;
-
-    case ustring_type:
-        /* Correct for Python v3. */
-        type_name = "bytes";
+        fprintf(fp, "%s.voidptr", sip_name);
         break;
 
     case string_type:
     case sstring_type:
+    case ustring_type:
+        type_name = "bytes";
+        break;
+
     case wstring_type:
     case ascii_string_type:
     case latin1_string_type:
@@ -1020,6 +1013,7 @@ static void pyiType(sipSpec *pt, moduleDef *mod, argDef *ad, int out,
     case cint_type:
     case ssize_type:
     case size_type:
+    case hash_type:
         type_name = "int";
         break;
 
@@ -1064,7 +1058,16 @@ static void pyiType(sipSpec *pt, moduleDef *mod, argDef *ad, int out,
         break;
 
     case pybuffer_type:
-        type_name = "sip.Buffer";
+        if (pep484)
+            fprintf(fp, "%s.Buffer", sip_name);
+        else
+            /* This replicates sip.pyi. */
+            fprintf(fp, "Union[bytes, bytearray, memoryview, %s.array, %s.voidptr]", sip_name, sip_name);
+
+        break;
+
+    case pyenum_type:
+        type_name = "enum.Enum";
         break;
 
     case ellipsis_type:
@@ -1515,15 +1518,13 @@ static int parseTypeHintNode(sipSpec *pt, int out, int top_level, char *start,
             break;
         }
 
-    /* We must have a name unless we have brackets. */
+    /* See if we have a name. */
     if (name_start == name_end)
     {
-        /* Check we have brackets. */
-        if (!have_brackets)
-            return FALSE;
-
-        /* Check we don't have empty brackets at the top level. */
-        if (top_level && children == NULL)
+        /*
+         * At the top level we must have brackets and they must not be empty.
+         */
+        if (top_level && (!have_brackets || children == NULL))
             return FALSE;
 
         /* Return the representation of brackets. */
@@ -1835,7 +1836,7 @@ static typeHintNodeDef *copyTypeHintNode(sipSpec *pt, typeHintDef *thd,
 
 
 /*
- * Lookup an enum.
+ * Lookup an enum using its C/C++ name.
  */
 static enumDef *lookupEnum(sipSpec *pt, const char *name, classDef *scope_cd,
         mappedTypeDef *scope_mtd)
@@ -1843,7 +1844,7 @@ static enumDef *lookupEnum(sipSpec *pt, const char *name, classDef *scope_cd,
     enumDef *ed;
 
     for (ed = pt->enums; ed != NULL; ed = ed->next)
-        if (ed->pyname != NULL && strcmp(ed->pyname->text, name) == 0 && ed->ecd == scope_cd && ed->emtd == scope_mtd)
+        if (ed->fqcname != NULL && strcmp(scopedNameTail(ed->fqcname), name) == 0 && ed->ecd == scope_cd && ed->emtd == scope_mtd)
             return ed;
 
     return NULL;
@@ -1851,166 +1852,35 @@ static enumDef *lookupEnum(sipSpec *pt, const char *name, classDef *scope_cd,
 
 
 /*
- * Lookup a mapped type.
+ * Lookup a mapped type using its C/C++ name.
  */
 static mappedTypeDef *lookupMappedType(sipSpec *pt, const char *name)
 {
     mappedTypeDef *mtd;
 
     for (mtd = pt->mappedtypes; mtd != NULL; mtd = mtd->next)
-        if (mtd->pyname != NULL && strcmp(mtd->pyname->text, name) == 0)
-        {
-            mappedTypeDef *impl = getMappedTypeImplementation(pt, mtd);
-
-            if (impl != NULL)
-                return impl;
-        }
+        if (mtd->cname != NULL && strcmp(mtd->cname->text, name) == 0)
+            return mtd;
 
     return NULL;
 }
 
 
 /*
- * Lookup a class.
+ * Lookup a class/struct/union using its C/C++ name.
  */
 static classDef *lookupClass(sipSpec *pt, const char *name, classDef *scope_cd)
 {
     classDef *cd;
 
     for (cd = pt->classes; cd != NULL; cd = cd->next)
-        if (strcmp(cd->pyname->text, name) == 0 && cd->ecd == scope_cd && !isExternal(cd))
-        {
-            classDef *impl = getClassImplementation(pt, cd);
-
-            if (impl != NULL)
-                return impl;
-        }
+        if (strcmp(classBaseName(cd), name) == 0 && cd->ecd == scope_cd && !isExternal(cd))
+            return cd;
 
     return NULL;
 }
 
 
-/*
- * Get the implementation (if there is one) for a type for the default API
- * version.
- */
-void getDefaultImplementation(sipSpec *pt, argType atype, classDef **cdp,
-        mappedTypeDef **mtdp)
-{
-    classDef *cd;
-    mappedTypeDef *mtd;
-    ifaceFileDef *iff;
-
-    if (atype == class_type)
-    {
-        cd = *cdp;
-        mtd = NULL;
-        iff = cd->iff;
-    }
-    else
-    {
-        cd = NULL;
-        mtd = *mtdp;
-        iff = mtd->iff;
-    }
-
-    /* See if there is more than one implementation. */
-    if (iff->api_range != NULL)
-    {
-        int def_api;
-
-        cd = NULL;
-        mtd = NULL;
-
-        /* Find the default implementation. */
-        def_api = findAPI(pt, iff->api_range->api_name->text)->from;
-
-        for (iff = iff->first_alt; iff != NULL; iff = iff->next_alt)
-        {
-            apiVersionRangeDef *avd = iff->api_range;
-
-            if (avd->from > 0 && avd->from > def_api)
-                continue;
-
-            if (avd->to > 0 && avd->to <= def_api)
-                continue;
-
-            /* It's within range. */
-            if (iff->type == class_iface)
-            {
-                for (cd = pt->classes; cd != NULL; cd = cd->next)
-                    if (cd->iff == iff)
-                        break;
-            }
-            else
-            {
-                for (mtd = pt->mappedtypes; mtd != NULL; mtd = mtd->next)
-                    if (mtd->iff == iff)
-                        break;
-            }
-
-            break;
-        }
-    }
-
-    *cdp = cd;
-    *mtdp = mtd;
-}
-
-
-/*
- * Return TRUE if a version range includes the default API.
- */
-int inDefaultAPI(sipSpec *pt, apiVersionRangeDef *range)
-{
-    int def_api;
-
-    /* Handle the trivial case. */
-    if (range == NULL)
-        return TRUE;
-
-    /* Get the default API. */
-    def_api = findAPI(pt, range->api_name->text)->from;
-
-    if (range->from > 0 && range->from > def_api)
-        return FALSE;
-
-    if (range->to > 0 && range->to <= def_api)
-        return FALSE;
-
-    return TRUE;
-}
-
-
-/*
- * Get the class implementation (if there is one) of the given class according
- * to the default version of any relevant API.
- */
-static classDef *getClassImplementation(sipSpec *pt, classDef *cd)
-{
-    mappedTypeDef *mtd;
-
-    getDefaultImplementation(pt, class_type, &cd, &mtd);
-
-    return cd;
-}
-
-
-/*
- * Get the mapped type implementation (if there is one) of the given mapped
- * type according to the default version of any relevant API.
- */
-static mappedTypeDef *getMappedTypeImplementation(sipSpec *pt,
-        mappedTypeDef *mtd)
-{
-    classDef *cd;
-
-    getDefaultImplementation(pt, mapped_type, &cd, &mtd);
-
-    return mtd;
-}
-
-
 /*
  * Generate a hint taking into account that it may be any sort of object.
  */
@@ -2018,3 +1888,29 @@ static void maybeAnyObject(const char *hint, int pep484, FILE *fp)
 {
     fprintf(fp, "%s", (strcmp(hint, "Any") != 0 ? hint : anyObject(pep484)));
 }
+
+
+/*
+ * Check if a word is a Python keyword (or has been at any time).
+ */
+static int isPyKeyword(const char *word)
+{
+    static const char *kwds[] = {
+        "False", "None", "True", "and", "as", "assert", "break", "class",
+        "continue", "def", "del", "elif", "else", "except", "finally", "for",
+        "from", "global", "if", "import", "in", "is", "lambda", "nonlocal",
+        "not", "or", "pass", "raise", "return", "try", "while", "with'"
+        "yield",
+        /* Historical keywords. */
+        "exec", "print",
+        NULL
+    };
+
+    const char **kwd;
+
+    for (kwd = kwds; *kwd != NULL; ++kwd)
+        if (strcmp(*kwd, word) == 0)
+            return TRUE;
+
+    return FALSE;
+}
