@@ -1,4 +1,4 @@
-# Copyright (c) 2021, Riverbank Computing Limited
+# Copyright (c) 2022, Riverbank Computing Limited
 # All rights reserved.
 #
 # This copy of SIP is licensed for use under the terms of the SIP License
@@ -21,13 +21,13 @@
 # POSSIBILITY OF SUCH DAMAGE.
 
 
-from collections import OrderedDict
-from distutils.sysconfig import get_python_inc, get_python_lib
+import collections
 import os
 import packaging
 import shutil
 import subprocess
 import sys
+import sysconfig
 import tempfile
 import warnings
 
@@ -37,7 +37,7 @@ from .bindings import Bindings
 from .configurable import Configurable, Option
 from .exceptions import UserException
 from .module import resolve_abi_version
-from .py_versions import FIRST_SUPPORTED_MINOR, LAST_SUPPORTED_MINOR
+from .py_versions import OLDEST_SUPPORTED_MINOR
 from .pyproject import (PyProjectException, PyProjectOptionException,
         PyProjectUndefinedOptionException)
 
@@ -78,7 +78,7 @@ class Project(AbstractProject, Configurable):
         Option('py_debug', option_type=bool),
 
         # The name of the directory containing Python.h.
-        Option('py_include_dir', default=get_python_inc()),
+        Option('py_include_dir', default=sysconfig.get_path('include')),
 
         # The name of the target Python platform.
         Option('py_platform'),
@@ -133,11 +133,18 @@ class Project(AbstractProject, Configurable):
         Option('scripts_dir', default=os.path.dirname(sys.executable),
                 help="the scripts installation directory", metavar="DIR",
                 tools=['build', 'install']),
-        Option('target_dir', default=get_python_lib(plat_specific=1),
+        Option('target_dir', default=sysconfig.get_path('platlib'),
                 help="the target installation directory", metavar="DIR",
                 tools=['build', 'install']),
         Option('api_dir', help="generate a QScintilla .api file in DIR",
                 metavar="DIR"),
+        Option('compile', option_type=bool, inverted=True,
+                help="disable the compilation of the generated code",
+                tools=['build']),
+        Option('version_info', option_type=bool, inverted=True,
+                help="disable any reference to the SIP version number in "
+                        "generated code",
+                tools=['build']),
     )
 
     # The configurable options for multiple bindings.
@@ -155,7 +162,7 @@ class Project(AbstractProject, Configurable):
 
         # The current directory should contain the .toml file.
         self.root_dir = os.getcwd()
-        self.bindings = OrderedDict()
+        self.bindings = collections.OrderedDict()
         self.bindings_factories = []
         self.builder = None
         self.buildables = []
@@ -177,18 +184,22 @@ class Project(AbstractProject, Configurable):
             self.bindings_factory = self.import_callable(self.bindings_factory,
                     Bindings)
 
+        if self.py_major_version is None or self.py_minor_version is None:
+            self.py_major_version = sys.hexversion >> 24
+            self.py_minor_version = (sys.hexversion >> 16) & 0x0ff
+
         if self.builder_factory is None:
-            from .distutils_builder import DistutilsBuilder
-            self.builder_factory = DistutilsBuilder
+            if (self.py_major_version, self.py_minor_version) >= (3, 10):
+                from .setuptools_builder import SetuptoolsBuilder
+                self.builder_factory = SetuptoolsBuilder
+            else:
+                from .distutils_builder import DistutilsBuilder
+                self.builder_factory = DistutilsBuilder
         elif isinstance(self.builder_factory, str):
             # Convert the name to a callable.
             self.builder_factory = self.import_callable(self.builder_factory,
                     AbstractBuilder)
 
-        if self.py_major_version is None or self.py_minor_version is None:
-            self.py_major_version = sys.hexversion >> 24
-            self.py_minor_version = (sys.hexversion >> 16) & 0x0ff
-
         if self.py_platform is None:
             self.py_platform = sys.platform
 
@@ -311,6 +322,47 @@ class Project(AbstractProject, Configurable):
 
         return ''
 
+    def get_platform_tag(self):
+        """ Return the platform tag to use in a wheel name.  This default
+        implementation uses the platform name and applies PEP defined
+        conventions depending on OS version and GLIBC version as appropriate.
+        """
+
+        platform_tag = sysconfig.get_platform()
+
+        if self.py_platform == 'darwin' and self.minimum_macos_version:
+            # We expect a three part tag so leave anything else unchanged.
+            parts = platform_tag.split('-')
+            if len(parts) == 3:
+                parts[1] = '{}.{}'.format(self.minimum_macos_version[0],
+                        self.minimum_macos_version[1])
+
+                platform_tag = '-'.join(parts)
+
+        elif self.py_platform == 'linux' and self.manylinux:
+            # We expect a two part tag so leave anything else unchanged.
+            parts = platform_tag.split('-')
+            if len(parts) == 2:
+                if self.minimum_glibc_version > (2, 17):
+                    # PEP 600.
+                    parts[0] = 'manylinux'
+                    parts.insert(1,
+                            '{}.{}'.format(self.minimum_glibc_version[0],
+                                    self.minimum_glibc_version[1]))
+                elif self.minimum_glibc_version > (2, 12):
+                    # PEP 599.
+                    parts[0] = 'manylinux2014'
+                elif self.minimum_glibc_version > (2, 5):
+                    # PEP 571.
+                    parts[0] = 'manylinux2010'
+                else:
+                    # PEP 513.
+                    parts[0] = 'manylinux1'
+
+                platform_tag = '-'.join(parts)
+
+        return platform_tag.replace('.', '_').replace('-', '_')
+
     def get_requires_dists(self):
         """ Return any 'Requires-Dist' to add to the project's meta-data. """
 
@@ -392,6 +444,64 @@ class Project(AbstractProject, Configurable):
         self.builder.install()
         self._remove_build_dir()
 
+    @property
+    def minimum_glibc_version(self):
+        """ The getter for the minimum GLIBC version. """
+
+        return self._minimum_glibc_version
+
+    @minimum_glibc_version.setter
+    def minimum_glibc_version(self, value):
+        """ The setter for the minimum GLIBC version. """
+
+        # Handle the initial creation of the option value.
+        if value is None and not hasattr(self, '_minimum_glibc_version'):
+            self._minimum_glibc_version = None
+            return
+
+        # Make sure any minimum GLIBC version is valid and convert it to a
+        # 2-tuple.
+        if value:
+            try:
+                value = self._convert_major_minor(value)
+            except ValueError:
+                raise PyProjectOptionException('minimum-glibc-version',
+                        "'{0}' is an invalid GLIBC version number".format(
+                                value),
+                        section_name='tool.sip.project')
+        else:
+            value = (2, 5)
+
+        self._minimum_glibc_version = value
+
+    @property
+    def minimum_macos_version(self):
+        """ The getter for the minimum macOS version. """
+
+        return self._minimum_macos_version
+
+    @minimum_macos_version.setter
+    def minimum_macos_version(self, value):
+        """ The setter for the minimum macOS version. """
+
+        # Handle the initial creation of the option value.
+        if value is None and not hasattr(self, '_minimum_macos_version'):
+            self._minimum_macos_version = None
+            return
+
+        # Make sure any minimum macOS version is valid and convert it to a
+        # 2-tuple.
+        if value:
+            try:
+                value = self._convert_major_minor(value)
+            except ValueError:
+                raise PyProjectOptionException('minimum-macos-version',
+                        "'{0}' is an invalid macOS version number".format(
+                                value),
+                        section_name='tool.sip.project')
+
+        self._minimum_macos_version = value
+
     @staticmethod
     def open_for_writing(fname):
         """ Open a file for writing while handling any errors. """
@@ -545,49 +655,25 @@ class Project(AbstractProject, Configurable):
                     "'{0}' must begin with a digit".format(self.build_tag),
                     section_name='tool.sip.project')
 
-        # Make sure any minimum GLIBC version is valid and convert it to a
-        # 2-tuple.
-        if self.minimum_glibc_version:
-            try:
-                self.minimum_glibc_version = self._convert_major_minor(
-                        self.minimum_glibc_version)
-            except ValueError:
-                raise PyProjectOptionException('minimum-glibc-version',
-                        "'{0}' is an invalid GLIBC version number".format(
-                                self.minimum_glibc_version),
-                        section_name='tool.sip.project')
-        else:
-            self.minimum_glibc_version = (2, 5)
-
-        # Make sure any minimum macOS version is valid and convert it to a
-        # 2-tuple.
-        if self.minimum_macos_version:
-            try:
-                self.minimum_macos_version = self._convert_major_minor(
-                        self.minimum_macos_version)
-            except ValueError:
-                raise PyProjectOptionException('minimum-macos-version',
-                        "'{0}' is an invalid macOS version number".format(
-                                self.minimum_macos_version),
-                        section_name='tool.sip.project')
-
         # Make sure relevent paths are absolute and use native separators.
         self.sip_files_dir = self.project_path(self.sip_files_dir)
         self.sip_include_dirs = [self.project_path(d)
                 for d in self.sip_include_dirs]
 
-        # Make sure we support the targeted version of Python.
+        # Check that the targeted version of Python isn't too old.  We hope
+        # that we will support newer versions automatically, but it's not
+        # guaranteed.
         py_version = (self.py_major_version, self.py_minor_version)
-        first_version = (3, FIRST_SUPPORTED_MINOR)
-        last_version = (3, LAST_SUPPORTED_MINOR)
+        first_version = (3, OLDEST_SUPPORTED_MINOR)
 
-        if py_version < first_version or py_version > last_version:
+        if py_version < first_version or self.py_major_version > 3:
             raise UserException(
                     "Python v{}.{} is not supported".format(
                             self.py_major_version, self.py_minor_version))
 
-        # Make sure we have a supported ABI version.
-        self.abi_version = resolve_abi_version(self.abi_version, exact=False)
+        # Get the supported ABI version.  (The actual version may have a later
+        # minor version.)
+        self.abi_version = resolve_abi_version(self.abi_version, module=False)
 
         # Checks for standalone projects.
         if tool in Option.BUILD_TOOLS and not self.sip_module:
